#include <iostream>
// for the vector class
#include <vector>
// And the list class
#include <list>

// vectors and lists are part of the STL
// so they're in the std namespace.
using namespace std;

 
// Some simple vector operations.
void vector_demo_1()
{
    // Let's make a vector of integers, but a vector
    // can hold ANY type.
    vector<int> vec_i ;
    
    // Type vec_i. and look at the list of method that show up.

    // Starts out with nothing loaded.  Add some stuff.
    vec_i.push_back(100) ;
    vec_i.push_back(200) ;
    // Print using index access
    cout << vec_i[0] << "  " << vec_i[1] << endl ;
    cout << "capacity: " << vec_i.capacity() << endl ;

    // A for loop that will add 10000 integers to the vector:
    for (int i = 0 ; i < 10000 ; ++i)
    {
        vec_i.push_back(i);
    }
    // Print out the number of elements in the vector
    cout << "count: " << vec_i.size() << endl ;
    // Print out the capacity, the number of elements that
    // it can store until it has to re-allocate memory (automatically)
    cout << "capacity: " << vec_i.capacity() << endl ;

    // What if we ask for something outside of the size?
    // Here the size is:  10002 and the capacity 16384.
    vec_i[10013] = 1;
    cout << "count: " << vec_i.size() << endl ;
    // Try accessing outside of the capacity.
    //vec_i[28000] =  1;
    // This may have caused a code crash. Or not.

    // Flush forces the console to print anything written to it.
    cout << flush ;
}

// Look at putting objects into a vector.
void vector_demo_2()
{
    // Here make a string vector.
    vector<string> vec_s ;

    // In a for loop, let's add a few more elements.
    for (int i = 0 ; i < 3 ; ++i)
    {
        string univ = "Boston" ;
        vec_s.push_back(univ) ;
        // string univ was COPIED into a new string held
        // in the vector. Proof: print out univ along with the element just added
        // to the vector.
        cout << vec_s[i] << " " << univ << endl ;
    }

    // In an alternate approach, make a string and then
    // move it into the vector.
    for (int i = 0 ; i < 3 ; ++i)
    {
        string univ = "Boston" ;
        // Use the C++11 move command to assign ownership of the
        // string to the vector. Another method, emplace_back,
        // handles the move for you if it can.
        vec_s.push_back(std::move(univ)) ;

        // univ is now back to its default value of an empty string.  No copying was
        // done.
        cout << vec_s[i] << " " << univ << endl ;
    }

    // When vec_s goes out of scope, what happens to the strings
    // in it?
}

// Using a different container
void vector_demo_3()
{
    // Make a list and add some things to it.
    list<float> lst ;
    for (int i = 0 ; i < 10 ; ++i) {
        lst.push_back(i-5.0) ;
    }
    // Make another list using the copy constructor
    list<float> lst_2(lst) ;

    // And a third using the assignment operator.
    list<float> lst_3 = lst ;
    
    // Will this work?  Create a vector from the list. Uncomment and try it.
    //vector<float> vec_test(lst) ;
    
    // How about this?
    vector<float> vec_test(lst.begin(),lst.end());
    // Print out the result.
//    for (int i = 0 ; i < vec_test.size() ; ++i) {
//        cout << vec_test[i] << "  " << lst[i] << endl ;
//    }
}



int main()
{
    cout << "" ;
    // 1st run the basic vector demo.
    vector_demo_1() ;

    // 2nd vector demo with some strings.
    vector_demo_2() ;

    // 3rd vector demo comparing with lists.
    vector_demo_3() ;

    return 0;
}
